import React from 'react'
import { get, last } from 'lodash'
import { compose, withProps } from 'recompose'
import styled from 'styled-components'
import { reduxForm } from 'redux-form'
import { th } from '@pubsweet/ui-toolkit'
import { required } from 'xpub-validators'
import { Button, Menu, ValidatedField } from '@pubsweet/ui'
import { withModal } from 'pubsweet-component-modal/src/components'

import {
  Row,
  Item,
  Label,
  Textarea,
  MultiAction,
  ContextualBox,
  ItemOverrideAlert,
  withFetching,
} from '../'

const eicDecisions = [
  {
    value: 'return-to-handling-editor',
    label: 'Return to Handling Editor',
    modalTitle: 'Return Manuscript',
    modalSubtitle:
      'A returning manuscript to Handling Editor decision is final',
  },
  {
    value: 'publish',
    label: 'Publish',
    modalTitle: 'Publish Manuscript',
    modalSubtitle: 'A publish decision is final',
  },
  {
    value: 'reject',
    label: 'Reject',
    modalTitle: 'Reject Manuscript',
    modalSubtitle: 'A rejection decision is final',
  },
]

const ManuscriptEicDecision = ({
  isFetching,
  formValues,
  handleSubmit,
  messagesLabel,
  collection = {},
  ...rest
}) => (
  <ContextualBox label="Your Editorial Decision" {...rest}>
    <Root>
      <Row justify="flex-start">
        <ItemOverrideAlert flex={0} vertical>
          <Label required>Decision</Label>
          <ValidatedField
            component={input => (
              <Menu
                {...input}
                options={
                  get(collection, 'status', 'submitted') === 'submitted'
                    ? [last(eicDecisions)]
                    : eicDecisions
                }
              />
            )}
            name="decision"
            validate={[required]}
          />
        </ItemOverrideAlert>
      </Row>

      {get(formValues, 'decision') !== 'publish' && (
        <Row mt={2}>
          <Item vertical>
            <Label required>
              {messagesLabel[get(formValues, 'decision', 'reject')]}
            </Label>
            <ValidatedField
              component={ValidatedTextArea}
              name="message"
              validate={[required]}
            />
          </Item>
        </Row>
      )}

      <Row justify="flex-end" mt={4}>
        <Button onClick={handleSubmit} primary size="medium">
          SUBMIT DECISION
        </Button>
      </Row>
    </Root>
  </ContextualBox>
)

export default compose(
  withFetching,
  withModal(({ isFetching }) => ({
    isFetching,
    modalKey: 'eic-decision',
    modalComponent: MultiAction,
  })),
  withProps(({ formValues }) => ({
    modalTitle: eicDecisions.find(
      o => o.value === get(formValues, 'decision', 'publish'),
    ).modalTitle,
    modalSubtitle: eicDecisions.find(
      o => o.value === get(formValues, 'decision', 'publish'),
    ).modalSubtitle,
  })),
  reduxForm({
    form: 'eic-decision',
    onSubmit: (
      values,
      dispatch,
      {
        submitDecision,
        showModal,
        setFetching,
        modalTitle,
        confirmMessage,
        modalSubtitle,
      },
    ) => {
      showModal({
        title: `${modalTitle}?`,
        confirmText: modalTitle,
        subtitle: modalSubtitle,
        onConfirm: modalProps => {
          submitDecision(values, { ...modalProps, setFetching })
        },
      })
    },
  }),
)(ManuscriptEicDecision)

// #region styles
const Root = styled.div`
  display: flex;
  flex-direction: column;
  padding: ${th('gridUnit')};
`

const ValidatedTextArea = styled(Textarea)`
  & + div {
    margin-top: 0;
  }
`
// #endregion
